function [xopt, topt] = half_bisection(r,n_plus,n_minus,L)
% function for solving the combinatorial feasibility problem via bisection,
% see problem (35) in F. Altenbach, G. Böcherer, R. Mathar, Short Huffman
% Codes Producing 1s Half of the Time. half_bisection is called by halfhc.
%
% INPUT
%   r       : pmf of codeword lengths
%   n_plus  : vector of maximum expected number of 1s
%   n_minus : vector of minimum expected number of 1s
%   L       : average length
%
% OUTPUT
%   xopt    : optimal selection vector
%   topt    : optimal value, corresponds to |q-0.5|
%
%   See also half_exhaustive, halfhc, huffmandict
%
% tested for Matlab R2010b
% 22 July 2011
% (c) Georg Böcherer, Fabian Altenbach
% email: geborg@gmail.com, fabian.altenbach@googlemail.com

% initialization for bisection
c = 1/L*r.*(n_minus - n_plus); b = 1/L * n_plus'*r - 1/2;
Atil = [-c'; c']; 
% frequency of 1s is at most b
% use b as upper bound
% if b negative, xopt = 0
xopt = zeros(length(r),1);
low = 0; up = b; 
epsil = 10^(-15);
num_notconverged = 0; num_iter = 0;
% bisection
while ((up - low) > epsil && num_iter < 10^2 )
    t = (low + up)/2;
    btil = [t + b; t - b];
    %adjust objective precision
    options = optimset('bintprog'); options.TolRLPFun = 1e-12;
    % solve the combinatorial feasibility problem
    [xlp,~,exitflag] = bintprog([],Atil,btil,[],[],[],options);
    if exitflag == -2 % infeasible
        low = t;
    else %feasible
        if(exitflag ~= 1)
            num_notconverged = num_notconverged + 1;
        end
        xopt = xlp;
        up =  t; %abs(c'*xopt + b);
    end;
    num_iter = num_iter + 1;
end;
topt = up;
