function [half_dict half_x] = halfhc(dict,p,L,method)
% halfhc - half Huffman coding
%   half_dict = halfhc(dict,p,L) returns an optimal prefix-free source code
%   with the expected frequency of 1s close to 0.5. halfhc implements the
%   algorithm proposed in F. Altenbach, G. Böcherer, R. Mathar, Short
%   Huffman Codes Producing 1s Half of the Time.
%
%   dict is a codebook as returned by huffmandict(alphabet,p).
%   p is a column vector with the probabilities of the symbols in alphabet.
%   L is the average codeword length of dict as returned by
%     [dict,L] ) = huffmandict(alphabet,p).
%   method is a string and decides which optimization method should be
%     applied. Currently, 'exhaustive' calls the solver half_exhaustive and
%     'bisection' calls the solver half_bisection.
%
%   See also half_bisection, half_exhaustive, huffmandict
%
% tested for Matlab R2010b
% 22 July 2011
% (c) Georg Böcherer, Fabian Altenbach
% email: geborg@gmail.com, fabian.altenbach@googlemail.com


%% find parameters
n= length(p);
codeword_lengths = zeros(n,1);
num_ones = zeros(n,1);

for i=1:n
   codeword_lengths(i) = length(dict{i,2});
   num_ones(i) = sum(dict{i,2});
end

lmin = min(codeword_lengths);
lmax = max(codeword_lengths);

m = lmax - lmin + 1;
r = zeros(m,1);
Nplus = zeros(m,1);
Nminus = zeros(m,1);


for j=lmin:lmax
    j_index = j-lmin+1;
    r(j_index) = sum(p(codeword_lengths==j));
    
    if(r(j_index)==0)
        Nplus(j_index) = 0;
        Nminus(j_index) = 0;
    else
        r_j = p(codeword_lengths==j);
        r_j = r_j/sum(r_j);
        r_j = sort(r_j,'ascend');
        num_ones_j = num_ones(codeword_lengths==j);
        Nplus(j_index) = sort(num_ones_j,'ascend')'*r_j;
        Nminus(j_index) = sort(num_ones_j,'descend')'*r_j;
    end
end

%% Optimize
if(nargin==3)
    method = 'exhaustive';
end

switch method
    case 'exhaustive'
        half_x = half_exhaustive(r,Nplus,Nminus,L);
    case 'bisection'
        half_x = half_bisection(r,Nplus,Nminus,L);
    otherwise
        half_x = half_exhaustive(r,Nplus,Nminus,L);
end
        
%% Apply Permutation
% calculate permutation
half_permutation = zeros(n,1);

for j=lmin:lmax
    j_index = j-lmin+1;
    symbol_index_j = get_index(codeword_lengths==j);
    
    if(r(j_index)==0)
        continue;
    else
        num_ones_j = num_ones(symbol_index_j);
        if(half_x(j_index)==0)%Nplus
            [~,ones_index] = sort(num_ones_j,'ascend');
        else%Nminus
            [~,ones_index] = sort(num_ones_j,'descend');
        end
        p_j = p(symbol_index_j);
        [~,p_index] = sort(p_j,'ascend');
        [~,p_nindex] = sort(p_index,'ascend');
        half_permutation(symbol_index_j) = symbol_index_j(ones_index(p_nindex));
    end
end

% apply permutation
half_dict = dict;

for i=1:n
   half_dict{i,2} = dict{half_permutation(i),2}; 
end

function index = get_index(boolean_index)
m = sum(boolean_index);
index=zeros(m,1);
j = 1;
for k=1:length(boolean_index)
    if(boolean_index(k)==true)
        index(j) = k;
        j = j+1;
    end
end
